/**-----------------------------------------------------------------------------

  Name         :  i2c_stm32f1.c
  Description  :  i2c driver for STM32F1 microcontroller
  Author       :  Aleksander Borysiuk

  History      :  2017.03.18 - Start of the project
                  2017.03.26 - Driver ver.1.0
                  2017.06.08 - Static i2c data buffer substituted by pointer
                               to external data buffer
                  2017.06.12 - Removed unnecessary data OR wih BSRR and BRR
                               GPIO registers during BSRR/BRR register write
                             - Driver ver.1.1
                  2017.07.15 - Added delay for i2c controller reset
                             - Added reset of the i2c controller if problem with
                               sending START condition is detected
                             - Drvier ver.1.2

------------------------------------------------------------------------------*/

/**-----------------------------------------------------------------------------
  Hardware info:
  This driver services I2C bus using STM32F1 microcontroller I2C interface (I2C1
  or I2C2) in polling fashion. The I2C bus is serviced in master mode providing
  burst write and burst read data to/from slave devices connected to the bus.

------------------------------------------------------------------------------*/

/* Includes ------------------------------------------------------------------*/
#include "i2c_stm32f1.h"
#include "stm32_BitBand.h"

/* Private define ------------------------------------------------------------*/
// I2C non calibrated timeouts (ca 10 and 100 I2C clock pulses respectively)
#define I2C_TIMEOUT          (10 * 72000000/I2C_BUS_FREQ)
#define I2C_LONG_TIMEOUT     (10 * I2C_TIMEOUT)

/* Private constants ---------------------------------------------------------*/

/* Private variables ---------------------------------------------------------*/
// - I2C error flags
uint8_t i2c_errFlags;
#define F_errSTART           BITBAND_SRAM(i2c_errFlags, 0)

/* Private function prototypes -----------------------------------------------*/

/* Private functions ---------------------------------------------------------*/

/* Public functions ----------------------------------------------------------*/
/**-----------------------------------------------------------------------------
  Name         :  i2c_Init
  Description  :  Performs I2C controller & MCI IO initialization
  Argument(s)  :  none
  Return value :  none
------------------------------------------------------------------------------*/
void i2c_Init(void)
{
  __IO uint32_t scl_time;              // SCL pulse timer
  uint32_t scl_cntr;                   // SCL pulses counter

  /// I2C pins configuration
  // - enable GPIO clock
  #if (I2C_INTFC_NO == 1)              // if I2C1 peripheral
    RCC->APB2ENR |= I2C_GPIO_CLK | RCC_APB2ENR_AFIOEN;
  #else                                // if I2C2 peripheral
    RCC->APB2ENR |= I2C_GPIO_CLK;
  #endif

  // - set mapping of SDA and SCL lines (I2C1 peripheral only!)
  #if ((I2C_INTFC_NO == 1) && \
       (I2C_GPIO_REMAP == 0))          // if default i2c pins
    AFIO->MAPR &= ~AFIO_MAPR_I2C1_REMAP;
  #elif ((I2C_INTFC_NO == 1) && \
         (I2C_GPIO_REMAP == 1))        // if i2c pins remapped
    AFIO->MAPR |= AFIO_MAPR_I2C1_REMAP;
  #endif

  // - clear GPIO CNF and MODE bit fields
  I2C_GPIO->CRH &= (~(((uint64_t)(GPIO_CRL_CNF0 | GPIO_CRL_MODE0) << 4*I2C_SDA) |
                      ((uint64_t)(GPIO_CRL_CNF0 | GPIO_CRL_MODE0) << 4*I2C_SCL)) >> 32);
  I2C_GPIO->CRL &= (~(((uint64_t)(GPIO_CRL_CNF0 | GPIO_CRL_MODE0) << 4*I2C_SDA) |
                      ((uint64_t)(GPIO_CRL_CNF0 | GPIO_CRL_MODE0) << 4*I2C_SCL)) >> 0);

  // - set SDA and SCL pins as 2MHz general purpose open drain outputs (i.e. conf.=0x1,
  //   mode=0x2), high level
  I2C_GPIO->BSRR = (1 << I2C_SDA) | (1 << I2C_SCL);
  I2C_GPIO->CRH |= ((((uint64_t)(GPIO_CRL_CNF0_0 | GPIO_CRL_MODE0_1) << 4*I2C_SDA) |
                     ((uint64_t)(GPIO_CRL_CNF0_0 | GPIO_CRL_MODE0_1) << 4*I2C_SCL)) >> 32);
  I2C_GPIO->CRL |= ((((uint64_t)(GPIO_CRL_CNF0_0 | GPIO_CRL_MODE0_1) << 4*I2C_SDA) |
                     ((uint64_t)(GPIO_CRL_CNF0_0 | GPIO_CRL_MODE0_1) << 4*I2C_SCL)) >> 0);

  // - reset stalled I2C bus
  //   When I2C controller is reset in the middle of its communication with the
  //   slave device, the slave device may still be waiting for the end of data
  //   exchange, blocking I2C bus. Application note AN-686 from Analog Devices
  //   suggests that such stalled I2C bus may be reset to iddle state by sending
  //   to the slave device remaining clock pulses (max. 9 pulses, i.e. 8 data
  //   bits + ACK bit) and then sending STOP condition.

  if ((I2C_GPIO->IDR & (1 << I2C_SDA)) == RESET){          // if i2c bus is stalled
                                                           // (i.e. SDA=low)
    scl_cntr = 0;                                          // init clock pulses counter
    while (((I2C_GPIO->IDR & (1 << I2C_SDA)) == RESET) &&  // while i2c bus is stalled
           (scl_cntr < 9)){                                // and there are clock pulses
                                                           // to be send

      I2C_GPIO->BRR = (1 << I2C_SCL);                      // SCL=low
      scl_time = I2C_TIMEOUT / 20;                         // clock low pulse time
      while (scl_time--);

      I2C_GPIO->BSRR = (1 << I2C_SCL);                     // SCL=high
      scl_time = I2C_TIMEOUT / 20;                         // clock high pulse time
      while (scl_time--);

      scl_cntr++;                                          // count clock pulses
    }

    // --- send STOP condition
    I2C_GPIO->BRR = (1 << I2C_SDA);                        // SDA=low
    scl_time = I2C_TIMEOUT / 20;                           // SDA change delay time
    while (scl_time--);
    I2C_GPIO->BSRR = (1 << I2C_SDA);                       // SDA=high
  }

  // - set SDA and SCL as 2MHz alternate function outputs, open drain (i.e. conf.=0x03, mode=0x02)
  I2C_GPIO->CRH |= ((((uint64_t)(GPIO_CRL_CNF0_1 | GPIO_CRL_CNF0_0 | GPIO_CRL_MODE0_1) << 4*I2C_SDA) |
                     ((uint64_t)(GPIO_CRL_CNF0_1 | GPIO_CRL_CNF0_0 | GPIO_CRL_MODE0_1) << 4*I2C_SCL)) >> 32);
  I2C_GPIO->CRL |= ((((uint64_t)(GPIO_CRL_CNF0_1 | GPIO_CRL_CNF0_0 | GPIO_CRL_MODE0_1) << 4*I2C_SDA) |
                     ((uint64_t)(GPIO_CRL_CNF0_1 | GPIO_CRL_CNF0_0 | GPIO_CRL_MODE0_1) << 4*I2C_SCL)) >> 0);


  /// I2C interface configuration
  // - enable I2C clock
  RCC->APB1ENR |= I2C_INTFC_CLK;

  // - disable I2C interface for configuration
  I2C_INTFC->CR1 &= ~I2C_CR1_PE;

  // - software reset I2C controller
  I2C_INTFC->CR1 |= I2C_CR1_SWRST;
  scl_time = I2C_TIMEOUT / 20;                             // i2c controller reset time
  while (scl_time--);
  I2C_INTFC->CR1 &= ~I2C_CR1_SWRST;

  // - configure I2C controller
  //   - SMBUS mode not used
  //   - I2C master transmitter / receiver
  //   - I2C mode active (SMBUS = 0)
  I2C_INTFC->CR1 = 0x00;

  // - configure DMA, interrupts and peripheral clock frequency
  //   - DMA transfers not used
  //   - interrupts not used
  //   - peripheral clock frequency equals to APB1 clock frequency
  I2C_INTFC->CR2 = (I2C_INTFC_FREQ / 1000000) & I2C_CR2_FREQ;

  // - configure I2C bus clock frequency generator and SCL rise time
  #if (I2C_BUS_FREQ <= 100000)
  // -- standard I2C mode: F/S = 0 (fSCL = 100kHz max), tLOW = tHIGH
  //   (tLOW = 4,7us min, tHIGH = 4,0us min), tRISE = 1000ns max
    I2C_INTFC->CCR = (I2C_INTFC_FREQ / I2C_BUS_FREQ / 2) & I2C_CCR_CCR;
    I2C_INTFC->TRISE = (I2C_INTFC_FREQ / 1000000 + 1) & I2C_TRISE_TRISE;
  #else
  // -- fast I2C mode: F/S = 1 (fSCL = 400kHz max), DUTY = 0 (tLOW = 2*tHIGH,
  //    tLOW = 1,2us min, tHIGH = 0,8us min), tRISE = 250ns max
    I2C_INTFC->CCR = I2C_CCR_FS | ((I2C_INTFC_FREQ / I2C_BUS_FREQ / 3) & I2C_CCR_CCR);
    I2C_INTFC->TRISE = (I2C_INTFC_FREQ / 4000000 + 1) & I2C_TRISE_TRISE;
  #endif

  // - enable I2C interface
  I2C_INTFC->CR1 |= I2C_CR1_PE;
}


/**-----------------------------------------------------------------------------
  Name         :  i2c_WriteData
  Description  :  Performs burst write data from data buffer to I2C device
  Argument(s)  :  pI2C_data - address of written data structure
  Return value :  SUCCESS if success or ERROR if fail
------------------------------------------------------------------------------*/
uint8_t i2c_WriteData(i2c_DataTypeDef *pI2C_data)
{
  uint32_t timeout;                    // I2C bus timeout counter
  uint32_t i;                          // loop counter

  /// Check if I2C controller is blocked
  // I2C controllers in STM32F1 microcontroller have an issue if STOP condition
  // was detected in the middle of data transfer (e.g. as a result of I2C bus noise).
  // In this case I2C controller blocks and is unable to send START condition in
  // master mode. STM32F105/107 Errata sheet recommends in such case resetting
  // I2C controller.
  if (F_errSTART){
    F_errSTART = 0;
    i2c_Init();                        // reset and initialize I2C controller
  }

  /// Check if I2C bus is free
  timeout = I2C_LONG_TIMEOUT;
  while (I2C_INTFC->SR2 & I2C_SR2_BUSY){
    timeout--;
    if (timeout == 0) return ERROR;
  }

  /// Send START condition
  I2C_INTFC->CR1 |= I2C_CR1_START;
  // - wait for the task end
  timeout = I2C_TIMEOUT;
  while ((I2C_INTFC->SR1 & I2C_SR1_SB) == RESET){
    timeout--;
    if (timeout == 0){
      F_errSTART = 1;                  // sending START condition failed
      return ERROR;
    }
  }

  /// Send I2C device address for write
  I2C_INTFC->DR = pI2C_data->device_addr & 0xFE;
  // - wait for the task end
  timeout = I2C_TIMEOUT;
  while ((I2C_INTFC->SR1 & I2C_SR1_ADDR) == RESET){
    timeout--;
    if (timeout == 0) return ERROR;
  }
  // - clear ADDR flag
  if ((I2C_INTFC->SR2 & I2C_SR2_TRA) == RESET) return ERROR;


  /// Send I2C device base register address
  I2C_INTFC->DR = pI2C_data->data_adr;
  // - wait for the task end
  timeout = I2C_TIMEOUT;
  while ((I2C_INTFC->SR1 & I2C_SR1_TXE) == RESET){
    timeout--;
    if (timeout == 0) return ERROR;
  }

  /// Send data
  for (i = 0; i < pI2C_data->data_size; i++){
    // - send data byte
    I2C_INTFC->DR = *(pI2C_data->pdata++);
    // - wait for the byte to be sent
    timeout = I2C_TIMEOUT;
    while ((I2C_INTFC->SR1 & I2C_SR1_TXE) == RESET){
      timeout--;
      if (timeout == 0) return ERROR;
    }
  }

  /// Wait for the last byte to be sent
  timeout = I2C_TIMEOUT;
  while ((I2C_INTFC->SR1 & I2C_SR1_BTF) == RESET){
    timeout--;
    if (timeout == 0) return ERROR;
  }

  /// Send STOP condition
  I2C_INTFC->CR1 |= I2C_CR1_STOP;

  return SUCCESS;
}


/**-----------------------------------------------------------------------------
  Name         :  i2c_ReadData
  Description  :  Performs burst read data from I2C device to data buffer
  Argument(s)  :  pI2C_data - address of read data structure
  Return value :  SUCCESS if success or ERROR if fail
------------------------------------------------------------------------------*/
uint8_t i2c_ReadData(i2c_DataTypeDef *pI2C_data)
{
  uint32_t timeout;                    // i2c bus timeout counter
  uint32_t i;                          // loop counter

  /// Check if I2C controller is blocked
  // I2C controllers in STM32F1 microcontroller have an issue if STOP condition
  // was detected in the middle of data transfer (e.g. as a result of I2C bus noise).
  // In this case I2C controller blocks and is unable to send START condition in
  // master mode. STM32F105/107 Errata sheet recommends in such case resetting
  // I2C controller.
  if (F_errSTART){
    F_errSTART = 0;
    i2c_Init();                        // reset and initialize I2C controller
  }

  /// Check if I2C bus is free
  timeout = I2C_LONG_TIMEOUT;
  while (I2C_INTFC->SR2 & I2C_SR2_BUSY){
    timeout--;
    if (timeout == 0) return ERROR;
  }

  /// Send START condition
  I2C_INTFC->CR1 |= I2C_CR1_START;
  // - wait for the task end
  timeout = I2C_TIMEOUT;
  while ((I2C_INTFC->SR1 & I2C_SR1_SB) == RESET){
    timeout--;
    if (timeout == 0){
      F_errSTART = 1;                  // sending START condition failed
      return ERROR;
    }
  }

  /// Send I2C device address for write
  I2C_INTFC->DR = pI2C_data->device_addr & 0xFE;
  // - wait for the task end
  timeout = I2C_TIMEOUT;
  while ((I2C_INTFC->SR1 & I2C_SR1_ADDR) == RESET){
    timeout--;
    if (timeout == 0) return ERROR;
  }
  // - clear ADDR flag
  if ((I2C_INTFC->SR2 & I2C_SR2_TRA) == RESET) return ERROR;

  /// Send I2C device base register address
  I2C_INTFC->DR = pI2C_data->data_adr;
  // - wait for the task end
  timeout = I2C_TIMEOUT;
  while ((I2C_INTFC->SR1 & I2C_SR1_BTF) == RESET){
    timeout--;
    if (timeout == 0) return ERROR;
  }

  /// Send START condition
  I2C_INTFC->CR1 |= I2C_CR1_START;
  // - wait for the task end
  timeout = I2C_TIMEOUT;
  while ((I2C_INTFC->SR1 & I2C_SR1_SB) == RESET){
    timeout--;
    if (timeout == 0){
      F_errSTART = 1;                  // sending START condition failed
      return ERROR;
    }
  }

  /// Send I2C device address for read
  I2C_INTFC->DR = pI2C_data->device_addr | 0x01;
  // - wait for the task end
  timeout = I2C_TIMEOUT;
  while ((I2C_INTFC->SR1 & I2C_SR1_ADDR) == RESET){
    timeout--;
    if (timeout == 0) return ERROR;
  }
  // - clear ADDR flag
  if ((I2C_INTFC->SR2 & I2C_SR2_TRA) != RESET) return ERROR;

  /// Receive data and send ACK
  for (i = 0; i < (pI2C_data->data_size - 1); i++){
    // - send ACK on data receive
    I2C_INTFC->CR1 |= I2C_CR1_ACK;
    // - wait for the task end
    timeout = I2C_TIMEOUT;
    while ((I2C_INTFC->SR1 & I2C_SR1_RXNE) == RESET){
      timeout--;
      if (timeout == 0) return ERROR;
    }
    // - read data byte
    *(pI2C_data->pdata++) = I2C_INTFC->DR;
  }

  /// Receive the last byte and send NAK
  // - send NAK on data receive
  I2C_INTFC->CR1 &= ~I2C_CR1_ACK;
  // - send STOP condition after data receive
  I2C_INTFC->CR1 |= I2C_CR1_STOP;
  // - wait for the task end
  timeout = I2C_TIMEOUT;
  while ((I2C_INTFC->SR1 & I2C_SR1_RXNE) == RESET){
    timeout--;
    if (timeout == 0) return ERROR;
  }
  // - read data byte
  *(pI2C_data->pdata++) = I2C_INTFC->DR;

  return SUCCESS;
}

/*----------------------------- End of file ----------------------------------*/
