/**-----------------------------------------------------------------------------

  Name         :  lcdHEM6432.c
  Description  :  HEM6432-03 64x32 graphic LCD driver for STM32F1 microcontroller
  Author       :  Aleksander Borysiuk

  History      :  2017.06.11 - Start of the project
                  2017.06.16 - LCD driver ver.1.0

------------------------------------------------------------------------------*/

/**-----------------------------------------------------------------------------
  Hardware info:
  HEM6432-03 (LUMMAX type LCD-AG-064032V-VAIW W/KKK-E6, former type LCD-AG-V6432)
  LCD module is a 0.49" graphic LCD display with resolution of 64x32 pixels. This
  LCD module has built in Sitronix ST7567A controller that drives LCD glass with
  1/33 duty and 1/5 bias. HEM6432 module configuration is very flexible. It may
  work either in normal (i.e. white characters on black background) or reversed
  (i.e. black characters on white background) mode, with normal display orientation
  (i.e. with module ribbon connector at the display's bottom) or upside down
  display orientation (i.e. with module ribbon connector at the display's top).
  All operation of HEM6432-03 LCD module is controlled via i2c interface working
  with clock frequency up to 400kHz.

  HEM6432-03 pinout
  HEM6432 LCD module is equipped with 14 pin ribbon edge connector intended
  for direct soldering to pcb.

                 ------------------------
                 |                      |
                 |        LCD           |
                 |    64 x 32 pixels    |
                 |                      |
                 ------------------------
                    |              |
                    |              |
                    |              |
                     --------------
                    |12345678911111|
                    |         01234|
                    ----------------

       1 : C2P(XVO) - DC/DC voltage converter
       2 : C2N(VO)  - DC/DC voltage converter
       3 : C1P(VG)  - DC/DC voltage converter
       4 : C1N(VSS) - DC/DC voltage converter
       5 : A        - supply voltage for backlight (+3.0V @15mA)
       6 : NC       - not connected
       7 : VSS(K)   - ground
       8 : VDD      - supply voltage (+2.7V..+3.3V)
       9 : RES#     - reset input
       10 : SCL     - serial clock input
       11 : SDA     - serial data input
       12 : NC      - not connected
       13 : NC      - not connected
       14 : NC      - not connected


  This software driver services HEM6432-03 LCD module via i2c interface. The
  driver uses two low-level functions: i2c_WriteData() and dwt_Delay() for
  data transfer and calibrated delay respectively. Both functions are provided
  by i2c_stm32f1 and dwt_utils peripheral drivers. Both peripherals (i.e. I2C
  and DWT) must be configured and running prior to use of this LCD driver!
------------------------------------------------------------------------------*/

/* Includes ------------------------------------------------------------------*/
#include "lcdHEM6432.h"
#include "i2c_stm32f1.h"
#include "dwt_utils.h"


/* Private define ------------------------------------------------------------*/
// ----------------------------------------------------------------------------
//               HEM6432 LCD module basic characteristics
// ----------------------------------------------------------------------------
// - LCD size
#define LCD_X_SIZE           64        // LCD horizontal size (i.e. 64 columns)
#define LCD_Y_SIZE           32        // LCD vertical size (i.e. 32 lines)

// - LCD i2c interface
#define LCD_I2C_ADDR         0x7E      // LCD i2c address
#define LCD_COMMAND_REG      0x00      // LCD command register address
#define LCD_DATA_REG         0x40      // LCD data register address

// - LCD DRAM
#ifndef LCD_UPSIDE_DOWN                // LCD module normal orientation
  #define LCD_COL_OFFSET     0x20      // LCD DRAM column address offset
#else                                  // LCD module upside down orientation
  #define LCD_COL_OFFSET     0x24      // LCD DRAM column address offset
#endif
#define LCD_PAGE_OFFSET      0x00      // LCD DRAM page number offset
#define LCD_PAGE_SIZE        8         // LCD DRAM page size (i.e. 8 lines)

// - LCD other defines
#define LCD_CONTRAST         0x1F      // LCD default contrast

// ----------------------------------------------------------------------------
//     HEM6432 LCD module control commands (controller: Sitronix ST7567A)
// ----------------------------------------------------------------------------
#define LCD_DISPLAY_OFF      0xAE      // display OFF
#define LCD_DISPLAY_ON       0xAF      // display ON

#define LCD_START_LINE       0x40      // set start line number (b5..b0: start
                                       // line number from 0 to 63)
#define LCD_START_LINE_MSK   0x3F      // start line number bit mask

#define LCD_PAGE_ADDR        0xB0      // set page address (b3..b0: page address
                                       // from 0 to 8)
#define LCD_PAGE_ADDR_MSK    0x0F      // page address bit mask

#define LCD_COL_ADDR_MSN     0x10      // set MSN column address (b3..b0: MSN
                                       // column address)
#define LCD_COL_ADDR_LSN     0x00      // set LSN column address (b3..b0: LSN
                                       // column address)
#define LCD_COL_ADDR_MSK     0x0F      // column address bit mask

#define LCD_SEG_DIR_NORMAL   0xA0      // set normal segment direction
#define LCD_SEG_DIR_REV      0xA1      // set reversed segment direction

#define LCD_DISPLAY_NORMAL   0xA6      // set normal display mode
#define LCD_DISPLAY_INV      0xA7      // set inversed display mode

#define LCD_ALL_PIXEL_OFF    0xA4      // switch-off ALL PIXEL ON mode
#define LCD_ALL_PIXEL_ON     0xA5      // switch-on ALL PIXEL ON mode

#define LCD_BIAS_LOW         0xA2      // set lcd bias:
                                       //   bias = 1/9 @ duty = 1/65
                                       //   bias = 1/8 @ duty = 1/49
                                       //   bias = 1/6 @ duty = 1/33
                                       //   bias = 1/8 @ duty = 1/55
#define LCD_BIAS_HIGH        0xA3      // set lcd bias:
                                       //   bias = 1/7 @ duty = 1/65
                                       //   bias = 1/6 @ duty = 1/49
                                       //   bias = 1/5 @ duty = 1/33
                                       //   bias = 1/6 @ duty = 1/55

#define LCD_R_W_ON           0xE0      // set Read-modify-Write mode
                                       // (column address increment: Read:+0 , Write:+1)
                                       //  (Not used with serial interface !)
#define LCD_R_W_END          0xEE      // exit Read-modify-Write mode
                                       // (Not used with serial interface !)

#define LCD_SOFT_RST         0xE2      // software reset

#define LCD_COM_DIR_NORMAL   0xC0      // set output COM direction normal
#define LCD_COM_DIR_REV      0xC8      // set output COM direction reversed

#define LCD_PWR_CONTROL      0x28      // lcd power control
                                       // b2..b0: power control OFF/ON
#define PWR_VB               0x04      //   voltage booster bit
#define PWR_VR               0x02      //   voltage regulator bit
#define PWR_VF               0x01      //   voltage follower bit

#define LCD_RR_3_0           0x20      // set resistor regulation ratio to 3.0
#define LCD_RR_3_5           0x21      // set resistor regulation ratio to 3.5
#define LCD_RR_4_0           0x22      // set resistor regulation ratio to 4.0
#define LCD_RR_4_5           0x23      // set resistor regulation ratio to 4.5
#define LCD_RR_5_0           0x24      // set resistor regulation ratio to 5.0
#define LCD_RR_5_5           0x25      // set resistor regulation ratio to 5.5
#define LCD_RR_6_0           0x26      // set resistor regulation ratio to 6.0
#define LCD_RR_6_5           0x27      // set resistor regulation ratio to 6.5

#define LCD_EV               0x81      // set electronic volume level (2 byte command!)
                                       // second byte: EV level from 0 to 63
#define LCD_EV_MSK           0x3F      // electronic volume level bit mask

#define LCD_BOOSTER          0xF8      // set booster level (2 byte command!)
                                       // second byte:
#define BOOSTER_LVL_4X       0x00      //   booster level 4x
#define BOOSTER_LVL_5X       0x01      //   booster level 5x

#define LCD_NOP              0xE3      // no operation

#define LCD_TEST             0xFE      // test (Do NOT use! Reserved for testing!)


/* Private constants ---------------------------------------------------------*/
// HEM6432-03 LCD module full initialization data
// (This table defines LCD module initialization code after hardware reset)
static const uint8_t lcdInitData[]=
{
  LCD_BIAS_HIGH,                        // bias select: 1/5 bias @ 1/33 duty
#ifndef LCD_UPSIDE_DOWN
// LCD module normal orientation
  LCD_SEG_DIR_NORMAL,                  // segment direction: normal
  LCD_COM_DIR_REV,                     // COM direction: reversed
#else
// LCD module upside down orientation
  LCD_SEG_DIR_REV,                     // segment direction: reversed
  LCD_COM_DIR_NORMAL,                  // COM direction: normal
#endif
  LCD_START_LINE,                      // start line number: line 0
  LCD_RR_5_5,                          // resistor regulation ratio: 5.5
  LCD_EV,                              // electronic volume level (2B command!)
  LCD_CONTRAST & LCD_EV_MSK,           //   EV / contrast
  LCD_BOOSTER,                         // booster level (2B command!)
  BOOSTER_LVL_5X,                      //   booster level 5x
  LCD_PWR_CONTROL|PWR_VB|PWR_VR|PWR_VF,// power control: voltage booster ON,
                                       // voltage regulator ON, voltage follower ON
  LCD_DISPLAY_NORMAL,                  // display mode: normal
  LCD_DISPLAY_ON                       // display ON
};


// HEM6432-03 LCD module refresh initialization data
// (This table defines LCD module initialization code after software reset)
static const uint8_t lcdRfshCfgData[]=
{
  LCD_SOFT_RST,                        // software reset
  LCD_BIAS_HIGH,                       // bias select: 1/5 bias @ 1/33 duty
#ifndef LCD_UPSIDE_DOWN
// LCD module normal orientation
  LCD_SEG_DIR_NORMAL,                  // segment direction: normal
  LCD_COM_DIR_REV,                     // COM direction: reversed
#else
// LCD module upside down orientation
  LCD_SEG_DIR_REV,                     // segment direction: reversed
  LCD_COM_DIR_NORMAL,                  // COM direction: normal
#endif
  LCD_RR_5_5,                          // resistor regulation ratio: 5.5
  LCD_EV,                              // electronic volume level (2B command!)
  LCD_CONTRAST & LCD_EV_MSK,           //   EV / contrast
  LCD_BOOSTER,                         // booster level (2B command!)
  BOOSTER_LVL_5X,                      //   booster level 5x
  LCD_PWR_CONTROL|PWR_VB|PWR_VR|PWR_VF,// power control: voltage booster ON,
                                       // voltage regulator ON, voltage follower ON
  LCD_DISPLAY_ON                       // display ON
};


// Character generator
// (This table defines the standard ASCII characters in a 5x7 dot format)
static const uint8_t LCDFontROM [][5] =
{
    { 0x00, 0x00, 0x00, 0x00, 0x00 },   // sp
    { 0x00, 0x00, 0x2f, 0x00, 0x00 },   // !
    { 0x00, 0x07, 0x00, 0x07, 0x00 },   // "
    { 0x14, 0x7f, 0x14, 0x7f, 0x14 },   // #
    { 0x24, 0x2a, 0x7f, 0x2a, 0x12 },   // $
    { 0xC4, 0xC8, 0x10, 0x26, 0x46 },   // %
    { 0x36, 0x49, 0x55, 0x22, 0x50 },   // &
    { 0x00, 0x05, 0x03, 0x00, 0x00 },   // '
    { 0x00, 0x1c, 0x22, 0x41, 0x00 },   // (
    { 0x00, 0x41, 0x22, 0x1c, 0x00 },   // )
    { 0x14, 0x08, 0x3E, 0x08, 0x14 },   // *
    { 0x08, 0x08, 0x3E, 0x08, 0x08 },   // +
    { 0x00, 0x00, 0x50, 0x30, 0x00 },   // ,
    { 0x10, 0x10, 0x10, 0x10, 0x10 },   // -
    { 0x00, 0x60, 0x60, 0x00, 0x00 },   // .
    { 0x20, 0x10, 0x08, 0x04, 0x02 },   // /
    { 0x3E, 0x51, 0x49, 0x45, 0x3E },   // 0
    { 0x00, 0x42, 0x7F, 0x40, 0x00 },   // 1
    { 0x42, 0x61, 0x51, 0x49, 0x46 },   // 2
    { 0x21, 0x41, 0x45, 0x4B, 0x31 },   // 3
    { 0x18, 0x14, 0x12, 0x7F, 0x10 },   // 4
    { 0x27, 0x45, 0x45, 0x45, 0x39 },   // 5
    { 0x3C, 0x4A, 0x49, 0x49, 0x30 },   // 6
    { 0x01, 0x71, 0x09, 0x05, 0x03 },   // 7
    { 0x36, 0x49, 0x49, 0x49, 0x36 },   // 8
    { 0x06, 0x49, 0x49, 0x29, 0x1E },   // 9
    { 0x00, 0x36, 0x36, 0x00, 0x00 },   // :
    { 0x00, 0x56, 0x36, 0x00, 0x00 },   // ;
    { 0x08, 0x14, 0x22, 0x41, 0x00 },   // <
    { 0x14, 0x14, 0x14, 0x14, 0x14 },   // =
    { 0x00, 0x41, 0x22, 0x14, 0x08 },   // >
    { 0x02, 0x01, 0x51, 0x09, 0x06 },   // ?
    { 0x32, 0x49, 0x59, 0x51, 0x3E },   // @
    { 0x7E, 0x11, 0x11, 0x11, 0x7E },   // A
    { 0x7F, 0x49, 0x49, 0x49, 0x36 },   // B
    { 0x3E, 0x41, 0x41, 0x41, 0x22 },   // C
    { 0x7F, 0x41, 0x41, 0x22, 0x1C },   // D
    { 0x7F, 0x49, 0x49, 0x49, 0x41 },   // E
    { 0x7F, 0x09, 0x09, 0x09, 0x01 },   // F
    { 0x3E, 0x41, 0x49, 0x49, 0x7A },   // G
    { 0x7F, 0x08, 0x08, 0x08, 0x7F },   // H
    { 0x00, 0x41, 0x7F, 0x41, 0x00 },   // I
    { 0x20, 0x40, 0x41, 0x3F, 0x01 },   // J
    { 0x7F, 0x08, 0x14, 0x22, 0x41 },   // K
    { 0x7F, 0x40, 0x40, 0x40, 0x40 },   // L
    { 0x7F, 0x02, 0x0C, 0x02, 0x7F },   // M
    { 0x7F, 0x04, 0x08, 0x10, 0x7F },   // N
    { 0x3E, 0x41, 0x41, 0x41, 0x3E },   // O
    { 0x7F, 0x09, 0x09, 0x09, 0x06 },   // P
    { 0x3E, 0x41, 0x51, 0x21, 0x5E },   // Q
    { 0x7F, 0x09, 0x19, 0x29, 0x46 },   // R
    { 0x46, 0x49, 0x49, 0x49, 0x31 },   // S
    { 0x01, 0x01, 0x7F, 0x01, 0x01 },   // T
    { 0x3F, 0x40, 0x40, 0x40, 0x3F },   // U
    { 0x1F, 0x20, 0x40, 0x20, 0x1F },   // V
    { 0x3F, 0x40, 0x38, 0x40, 0x3F },   // W
    { 0x63, 0x14, 0x08, 0x14, 0x63 },   // X
    { 0x07, 0x08, 0x70, 0x08, 0x07 },   // Y
    { 0x61, 0x51, 0x49, 0x45, 0x43 },   // Z
    { 0x00, 0x7F, 0x41, 0x41, 0x00 },   // [
    { 0x55, 0x2A, 0x55, 0x2A, 0x55 },   // 55
    { 0x00, 0x41, 0x41, 0x7F, 0x00 },   // ]
    { 0x04, 0x02, 0x01, 0x02, 0x04 },   // ^
    { 0x40, 0x40, 0x40, 0x40, 0x40 },   // _
    { 0x00, 0x01, 0x02, 0x04, 0x00 },   // '
    { 0x20, 0x54, 0x54, 0x54, 0x78 },   // a
    { 0x7F, 0x48, 0x44, 0x44, 0x38 },   // b
    { 0x38, 0x44, 0x44, 0x44, 0x20 },   // c
    { 0x38, 0x44, 0x44, 0x48, 0x7F },   // d
    { 0x38, 0x54, 0x54, 0x54, 0x18 },   // e
    { 0x08, 0x7E, 0x09, 0x01, 0x02 },   // f
    { 0x0C, 0x52, 0x52, 0x52, 0x3E },   // g
    { 0x7F, 0x08, 0x04, 0x04, 0x78 },   // h
    { 0x00, 0x44, 0x7D, 0x40, 0x00 },   // i
    { 0x20, 0x40, 0x44, 0x3D, 0x00 },   // j
    { 0x7F, 0x10, 0x28, 0x44, 0x00 },   // k
    { 0x00, 0x41, 0x7F, 0x40, 0x00 },   // l
    { 0x7C, 0x04, 0x18, 0x04, 0x78 },   // m
    { 0x7C, 0x08, 0x04, 0x04, 0x78 },   // n
    { 0x38, 0x44, 0x44, 0x44, 0x38 },   // o
    { 0x7C, 0x14, 0x14, 0x14, 0x08 },   // p
    { 0x08, 0x14, 0x14, 0x18, 0x7C },   // q
    { 0x7C, 0x08, 0x04, 0x04, 0x08 },   // r
    { 0x48, 0x54, 0x54, 0x54, 0x20 },   // s
    { 0x04, 0x3F, 0x44, 0x40, 0x20 },   // t
    { 0x3C, 0x40, 0x40, 0x20, 0x7C },   // u
    { 0x1C, 0x20, 0x40, 0x20, 0x1C },   // v
    { 0x3C, 0x40, 0x30, 0x40, 0x3C },   // w
    { 0x44, 0x28, 0x10, 0x28, 0x44 },   // x
    { 0x0C, 0x50, 0x50, 0x50, 0x3C },   // y
    { 0x44, 0x64, 0x54, 0x4C, 0x44 },   // z
    { 0x00, 0x08, 0x36, 0x41, 0x00 },   // {
    { 0x00, 0x00, 0x7f, 0x00, 0x00 },   // |
    { 0x00, 0x41, 0x36, 0x08, 0x00 },   // }
    { 0x02, 0x01, 0x02, 0x04, 0x02 }    // ~
};

/* Private variables ---------------------------------------------------------*/

/* Private function prototypes -----------------------------------------------*/

/* Private functions ---------------------------------------------------------*/

/* Public functions ----------------------------------------------------------*/
/**-----------------------------------------------------------------------------
  Name         :  lcdHEM6432_SendCmd
  Description  :  Sends command(s) to LCD
  Argument(s)  :  pCommandBuf - command buffer address
                  nByte - number of commands to send
  Return value :  none
------------------------------------------------------------------------------*/
void lcdHEM6432_SendCmd(uint8_t *pCommandBuf, uint8_t nByte)
{
  i2c_DataTypeDef lcdI2C;              // i2c data structure

  lcdI2C.device_addr = LCD_I2C_ADDR;   // LCD i2c address
  lcdI2C.data_adr = LCD_COMMAND_REG;   // LCD i2c command register
  lcdI2C.data_size = nByte;            // number of bytes to send
  lcdI2C.pdata = pCommandBuf;          // command buffer address

  i2c_WriteData(&lcdI2C);              // send commands to LCD
}


/**-----------------------------------------------------------------------------
  Name         :  lcdHEM6432_SendData
  Description  :  Sends display data to LCD
  Argument(s)  :  pDataBuf - display data buffer address
                  nByte - number of display data to send
  Return value :  none
------------------------------------------------------------------------------*/
void lcdHEM6432_SendData(uint8_t *pDataBuf, uint8_t nByte)
{
  i2c_DataTypeDef lcdI2C;              // i2c data structure

  lcdI2C.device_addr = LCD_I2C_ADDR;   // LCD i2c address
  lcdI2C.data_adr = LCD_DATA_REG;      // LCD i2c data register
  lcdI2C.data_size = nByte;            // number of bytes to send
  lcdI2C.pdata = pDataBuf;             // display data buffer address

  i2c_WriteData(&lcdI2C);              // send display data to LCD
}


/**-----------------------------------------------------------------------------
  Name         :  lcdHEM6432_Init
  Description  :  Performs MCU IO & LCD controller full initialization
  Argument(s)  :  none
  Return value :  none
------------------------------------------------------------------------------*/
void lcdHEM6432_Init(void)
{
  /// I2C and DWT configuration
  // Driver assumes that i2c peripheral and DWT unit are already configured
  // by peripheral drivers and runnung

  /// LCD_RST pin configuration
  // - enable GPIO clock
  RCC->APB2ENR |= LCD_RST_GPIO_CLK;

  // - clear GPIO CNF and MODE bit fields
  LCD_RST_GPIO->CRH &= (~((uint64_t)(GPIO_CRL_CNF0 | GPIO_CRL_MODE0) << 4*LCD_RST_PIN) >> 32);
  LCD_RST_GPIO->CRL &= (~((uint64_t)(GPIO_CRL_CNF0 | GPIO_CRL_MODE0) << 4*LCD_RST_PIN) >> 0);

  // - set LCD_RST pin as 2MHz general purpose push-pull output (i.e. conf.=0x0,
  //   mode=0x2), high level
  LCD_RST_GPIO->BSRR = (1 << LCD_RST_PIN);
  LCD_RST_GPIO->CRH |= (((uint64_t)(GPIO_CRL_MODE0_1) << 4*LCD_RST_PIN) >> 32);
  LCD_RST_GPIO->CRL |= (((uint64_t)(GPIO_CRL_MODE0_1) << 4*LCD_RST_PIN) >> 0);

  /// LCD hardware reset
  // LCD reset ought to be performed within 30ms after power supply voltage
  // becomes stable. Otherwise the display may operate in abnormal way. LCD
  // reset LOW pulse should last at least 1us and LCD should be reset within
  // 2us after rising edge of the reset pulse, but recommended delay times
  // should be longer than 5us and 5us respectively.
  LCD_RST_GPIO->BRR = (1 << LCD_RST_PIN);        // set LCD_RST line low
  dwt_Delay(10);                                 // wait for 10us
  LCD_RST_GPIO->BSRR = (1 << LCD_RST_PIN);       // set LCD_RST line high
  dwt_Delay(10);                                 // wait for 10us

  /// LCD initialization
  // LCD controller initialization code is stored in lcdInitData[] table.
  // Initialization phase, counting from hardware reset to setting Power Control,
  // ought to be completed within 5ms!
  lcdHEM6432_SendCmd((uint8_t *)lcdInitData, sizeof(lcdInitData));
}


/**-----------------------------------------------------------------------------
  Name         :  lcdHEM6432_RefreshCfg
  Description  :  Refresh LCD controller initialization
  Argument(s)  :  none
  Return value :  none
------------------------------------------------------------------------------*/
void lcdHEM6432_RefreshCfg(void)
{
  /// LCD initialization refresh
  // LCD controller refresh initialization code is stored in lcdRfshCfgData[]
  // table. It is avdiced to refresh controller initialization in regular
  // intervals during program operation.
  lcdHEM6432_SendCmd((uint8_t *)lcdRfshCfgData, sizeof(lcdRfshCfgData));
}


/**-----------------------------------------------------------------------------
  Name         :  lcdHEM6432_SetContrast
  Description  :  Sets LCD contrast
  Argument(s)  :  contrast - LCD contrast from 0 to 63
  Return value :  none
------------------------------------------------------------------------------*/
void lcdHEM6432_SetContrast(uint8_t contrast)
{
  uint8_t lcdContrastBuf[2];           // contrast buffer

  // Store 'Set EV' command code
  lcdContrastBuf[0] = LCD_EV;
  // Store contrast value (max. LCD_EV_MSK)
  lcdContrastBuf[1] = (contrast < LCD_EV_MSK ? contrast : LCD_EV_MSK);

  // Send command to LCD
  lcdHEM6432_SendCmd(lcdContrastBuf, 2);
}


/**-----------------------------------------------------------------------------
  Name         :  lcdHEM6432_SetNormalDisplay
  Description  :  Sets normal display mode (i.e. white chars on black background)
  Argument(s)  :  none
  Return value :  none
------------------------------------------------------------------------------*/
void lcdHEM6432_SetNormalDisplay(void)
{
  uint8_t lcdCommandBuf;               // command buffer

  // Store 'Normal display' command code
  lcdCommandBuf = LCD_DISPLAY_NORMAL;

  // Send command to LCD
  lcdHEM6432_SendCmd(&lcdCommandBuf, 1);
}


/**-----------------------------------------------------------------------------
  Name         :  lcdHEM6432_SetInvertedDisplay
  Description  :  Sets inverted display mode (i.e. black chars on white background)
  Argument(s)  :  none
  Return value :  none
------------------------------------------------------------------------------*/
void lcdHEM6432_SetInvertedDisplay(void)
{
  uint8_t lcdCommandBuf;               // command buffer

  // Store 'Inverted display' command code
  lcdCommandBuf = LCD_DISPLAY_INV;

  // Send command to LCD
  lcdHEM6432_SendCmd(&lcdCommandBuf, 1);
}


/**-----------------------------------------------------------------------------
  Name         :  lcdHEM6432_SetPowerSaveON
  Description  :  Sets display in power save mode
  Argument(s)  :  none
  Return value :  none
------------------------------------------------------------------------------*/
void lcdHEM6432_SetPowerSaveON(void)
{
  uint8_t lcdCommandBuf[2];            // command buffer

  // Store 'Display OFF' command code
  lcdCommandBuf[0] = LCD_DISPLAY_OFF;
  // Store 'All Pixel ON' command code
  lcdCommandBuf[1] = LCD_ALL_PIXEL_ON;

  // Send command to LCD
  lcdHEM6432_SendCmd(lcdCommandBuf, 2);
}


/**-----------------------------------------------------------------------------
  Name         :  lcdHEM6432_SetPowerSaveOFF
  Description  :  Sets display in normal power mode
  Argument(s)  :  none
  Return value :  none
------------------------------------------------------------------------------*/
void lcdHEM6432_SetPowerSaveOFF(void)
{
  uint8_t lcdCommandBuf[2];            // command buffer

  // Store 'All Pixel OFF' command code
  lcdCommandBuf[0] = LCD_ALL_PIXEL_OFF;
  // Store 'Display ON' command code
  lcdCommandBuf[1] = LCD_DISPLAY_ON;

  // Send command to LCD
  lcdHEM6432_SendCmd(lcdCommandBuf, 2);
}


/**-----------------------------------------------------------------------------
  Name         :  lcdHEM6432_GoToXY
  Description  :  Moves the cursor to position (X,Y)
  Argument(s)  :  Xpos - x coordinate 0..63
                  Ypos - y coordinate 0..3
  Return value :  none
------------------------------------------------------------------------------*/
void lcdHEM6432_GoToXY(uint8_t Xpos, uint8_t Ypos)
{
  uint8_t lcdPosBuf[3];                // cursor position buffer

  // Check x coordinate limit
  if (Xpos >= LCD_X_SIZE) Xpos = LCD_X_SIZE-1;
  // Convert Xpos to column number
  Xpos += LCD_COL_OFFSET;
  // Store 'Set column MSN address' command code with MSN column number
  lcdPosBuf[0] = LCD_COL_ADDR_MSN | (Xpos >> 4);
  // Store 'Set column LSN address' command code with LSN column number
  lcdPosBuf[1] = LCD_COL_ADDR_LSN | (Xpos & 0x0F);

  // Check y coordinate limit
  if (Ypos >= LCD_Y_SIZE/LCD_PAGE_SIZE) Ypos = LCD_Y_SIZE/LCD_PAGE_SIZE-1;
  // Store 'Set page address' command code with page number
  lcdPosBuf[2] = LCD_PAGE_ADDR | Ypos;

  // Send command to LCD
  lcdHEM6432_SendCmd(lcdPosBuf, 3);
}


/**-----------------------------------------------------------------------------
  Name         :  lcdHEM6432_Clean
  Description  :  Cleans the display and moves the cursor to position (0,0)
  Argument(s)  :  none
  Return value :  none
------------------------------------------------------------------------------*/
void lcdHEM6432_Clean(void)
{
  uint8_t i;                           // loop counter
  uint8_t lcdDataBuf[LCD_X_SIZE];      // data buffer

  // Clear data buffer
  for (i = 0; i < LCD_X_SIZE; i++)
  {
    lcdDataBuf[i] = 0x00;
  }

  // Clean LCD (each DRAM page is cleaned seperately)
  for (i = 0; i < LCD_Y_SIZE/LCD_PAGE_SIZE; i++)
  {
    // Move cursor to the beginning of DRAM page
    lcdHEM6432_GoToXY(0, i);
    // Clear DRAM page
    lcdHEM6432_SendData(lcdDataBuf, LCD_X_SIZE);
  }

  // Move cursor to position (0, 0)
  lcdHEM6432_GoToXY(0, 0);
}


/**-----------------------------------------------------------------------------
  Name         :  lcdHEM6432_PutChar
  Description  :  Displays character at current cursor location
  Argument(s)  :  char_code - character code (ASCII)
  Return value :  none
------------------------------------------------------------------------------*/
void lcdHEM6432_PutChar(uint8_t char_code)
{
  uint8_t i;                           // loop counter
  uint8_t lcdCharPatBuf[6];            // character pattern buffer

  // Check character code and convert it to the printable one
  if ((char_code < ' ') || (char_code > '~')) char_code = ' ';

  // Convert character code to char patterns ROM table index
  char_code -= ' ';

  // Copy character pattern to buffer
  for (i = 0; i < 5; i++)
  {
    lcdCharPatBuf[i] = LCDFontROM[char_code][i];
  }
  // Add one pixel space between characters
  lcdCharPatBuf[5] = 0x00;

  // Send character pattern to LCD
  lcdHEM6432_SendData(lcdCharPatBuf, 6);
}


/**-----------------------------------------------------------------------------
  Name         :  lcdHEM6432_PutText
  Description  :  Displays text string at current cursor location
  Argument(s)  :  text - text string address
  Return value :  none
------------------------------------------------------------------------------*/
void lcdHEM6432_PutText(uint8_t *text)
{
  while (*text != 0)                   // repeat till the end of the string
  {
    lcdHEM6432_PutChar(*text);         // display character
    text++;                            // next character
  }
}


/**-----------------------------------------------------------------------------
  Name         :  lcdHEM6432_PutBMP
  Description  :  Displays bitmap with size 64x32 pixels
  Argument(s)  :  pPicture - bitmap address
  Return value :  none
------------------------------------------------------------------------------*/
void lcdHEM6432_PutBMP(uint8_t *pPicture)
{
  uint8_t i;                           // loop counter

  // Send bitmap to LCD (each DRAM page is sent seperately)
  for (i = 0; i < LCD_Y_SIZE/LCD_PAGE_SIZE; i++)
  {
    // Move cursor to the beginning of DRAM page
    lcdHEM6432_GoToXY(0, i);
    // Send DRAM page
    lcdHEM6432_SendData(pPicture, LCD_X_SIZE);
    // Next page
    pPicture += LCD_X_SIZE;
  }
}
/*----------------------------- End of file ----------------------------------*/
