/**-----------------------------------------------------------------------------

  Name         :  main.h
  Description  :  HEM6432-03 LCD module demo
                  Program runs on STM32 Butterfly Board and displays simple demo
                  on HEM6432-03 LCD module. The LCD is connected to STM32 Butterfly
                  Board as follows:
                      VDD -> Con1 / pin9 (+3V3)
                      GND -> Con7 / pin4 (GND)
                      LED -> Con3 / pin9 (+3V3)
                      SDA -> Con7 / pin3 (PB7/SDA)
                      SCL -> Con7 / pin 2 (PB6/SCL)
                      RES -> Con1 / pin 7 (PB3)
                  Communication to LCD is serviced by I2C1 controller which works
                  in master mode with i2c bus clock frequency equal to 400kHz.
  Author       :  Aleksander Borysiuk
  History      :  2017.07.16 - ver.1.0

------------------------------------------------------------------------------*/

/* Includes ------------------------------------------------------------------*/
#include "stm32f10x.h"
#include "stm32_BitBand.h"
#include "stm32_Butterfly.h"
#include "dwt_utils.h"
#include "i2c_stm32f1.h"
#include "lcdHEM6432.h"


/* Private define ------------------------------------------------------------*/
#define SYS_CLK               72000000 // system clock 72MHz

// IRQ vectors location
#define NVIC_VectTab_RAM      ((uint32_t)0x20000000)
#define NVIC_VectTab_FLASH    ((uint32_t)0x08000000)
//#define IRQ_VectTab_RAM                // IRQ vectors in RAM (in FLASH if not defined)

// NVIC priority groups and subpriority levels
#define NVIC_PriorityGroup_0  ((uint32_t)0x7)    // 0 bits priority, 4 bits subpriority
#define NVIC_PriorityGroup_1  ((uint32_t)0x6)    // 1 bit  priority, 3 bits subpriority
#define NVIC_PriorityGroup_2  ((uint32_t)0x5)    // 2 bits priority, 2 bits subpriority
#define NVIC_PriorityGroup_3  ((uint32_t)0x4)    // 3 bits priority, 1 bit  subpriority
#define NVIC_PriorityGroup_4  ((uint32_t)0x3)    // 4 bits priority, 0 bits subpriority


/* Private macro -------------------------------------------------------------*/

/* Private variables ---------------------------------------------------------*/
static __IO uint32_t timingDelay;      // delay timer

static uint8_t sysFlags;               // system flags
#define F_500ms   BITBAND_SRAM(sysFlags, 0)
#define F_1s      BITBAND_SRAM(sysFlags, 1)

/* Private function prototypes -----------------------------------------------*/
void lcdDemo(void);
void Delay(__IO uint32_t nTime);


/* Private functions ---------------------------------------------------------*/
/**-----------------------------------------------------------------------------
  Name         :  main
  Description  :  Main program
  Argument(s)  :  none
  Return value :  none
------------------------------------------------------------------------------*/
int main(void)
{
  /* At this stage system clocks are already configured and running. This is
     done by SysInit() function which is called in startup code prior to
     branching to main() *****************************************************/

  /* Configure all unused GPIO port pins in Analog Input mode (floating input
     trigger OFF), this will reduce the power consumption and increase the device
     immunity against EMI/EMC *************************************************/
  RCC->APB2ENR |= (RCC_APB2ENR_IOPAEN | RCC_APB2ENR_IOPBEN | RCC_APB2ENR_IOPCEN |
                   RCC_APB2ENR_IOPDEN | RCC_APB2ENR_IOPEEN);

  // Ports PA14 and PA13 are used by SWD as SWCLK and SWDIO lines
  GPIOA->CRH &= ~(GPIO_CRH_CNF15 | GPIO_CRH_CNF12 | GPIO_CRH_CNF11 |
                  GPIO_CRH_CNF10 | GPIO_CRH_CNF9  | GPIO_CRH_CNF8);
  GPIOA->CRL &= ~GPIO_CRL_CNF;

  // Port PB3 is used by SWD as SWO line
  GPIOB->CRH &= ~GPIO_CRH_CNF;
  GPIOB->CRL &= ~(GPIO_CRL_CNF7 | GPIO_CRL_CNF6 | GPIO_CRL_CNF5 | GPIO_CRL_CNF4 |
                  GPIO_CRL_CNF2 | GPIO_CRL_CNF1 | GPIO_CRL_CNF0);

  GPIOC->CRH &= ~GPIO_CRH_CNF;
  GPIOC->CRL &= ~GPIO_CRL_CNF;

  GPIOD->CRH &= ~GPIO_CRH_CNF;
  GPIOD->CRL &= ~GPIO_CRL_CNF;

  GPIOE->CRH &= ~GPIO_CRH_CNF;
  GPIOE->CRL &= ~GPIO_CRL_CNF;

  RCC->APB2ENR &= ~(RCC_APB2ENR_IOPAEN | RCC_APB2ENR_IOPBEN | RCC_APB2ENR_IOPCEN |
                    RCC_APB2ENR_IOPDEN | RCC_APB2ENR_IOPEEN);


  // NVIC configuration
  // - set location of the vector interrupt table
#ifdef IRQ_VectTab_RAM
  SCB->VTOR = NVIC_VectTab_RAM;
#else
  SCB->VTOR = NVIC_VectTab_FLASH;
#endif
  // - set priority grouping: 8 group priorities + 2 subpriorities
  NVIC_SetPriorityGrouping(NVIC_PriorityGroup_3);


  // LED_1 and LED_2 pins configuration
  // - enable GPIO clock
  RCC->APB2ENR |= LED_GPIO_CLK;
  // - clear CNF and MODE bit fields
  LED_GPIO->CRH &= (~(((uint64_t)(GPIO_CRL_CNF0 | GPIO_CRL_MODE0) << 4*LED_1) |
                      ((uint64_t)(GPIO_CRL_CNF0 | GPIO_CRL_MODE0) << 4*LED_2)) >> 32);
  LED_GPIO->CRL &= (~(((uint64_t)(GPIO_CRL_CNF0 | GPIO_CRL_MODE0) << 4*LED_1) |
                      ((uint64_t)(GPIO_CRL_CNF0 | GPIO_CRL_MODE0) << 4*LED_2)) >> 0);
  // - set GPIO lines as 2MHz push-pull outputs (i.e. conf 0x00 & mode 0x02)
  LED_GPIO->CRH |= ((((uint64_t)GPIO_CRL_MODE0_1 << 4*LED_1) |
                     ((uint64_t)GPIO_CRL_MODE0_1 << 4*LED_2)) >> 32);
  LED_GPIO->CRL |= ((((uint64_t)GPIO_CRL_MODE0_1 << 4*LED_1) |
                     ((uint64_t)GPIO_CRL_MODE0_1 << 4*LED_2)) >> 0);
  // - switch off both LEDs
  LED_OFF(LED_1);
  LED_OFF(LED_2);


  // Joystick pins configuration
  // - enable GPIO clock
  RCC->APB2ENR |= JOY_GPIO_CLK;
  // - clear CNF and MODE bit fields
  JOY_GPIO->CRH &= (~(((uint64_t)(GPIO_CRL_CNF0 | GPIO_CRL_MODE0) << 4*JOY_OK) |
                      ((uint64_t)(GPIO_CRL_CNF0 | GPIO_CRL_MODE0) << 4*JOY_UP) |
                      ((uint64_t)(GPIO_CRL_CNF0 | GPIO_CRL_MODE0) << 4*JOY_DOWN) |
                      ((uint64_t)(GPIO_CRL_CNF0 | GPIO_CRL_MODE0) << 4*JOY_RIGHT) |
                      ((uint64_t)(GPIO_CRL_CNF0 | GPIO_CRL_MODE0) << 4*JOY_LEFT)) >> 32);
  JOY_GPIO->CRL &= (~(((uint64_t)(GPIO_CRL_CNF0 | GPIO_CRL_MODE0) << 4*JOY_OK) |
                      ((uint64_t)(GPIO_CRL_CNF0 | GPIO_CRL_MODE0) << 4*JOY_UP) |
                      ((uint64_t)(GPIO_CRL_CNF0 | GPIO_CRL_MODE0) << 4*JOY_DOWN) |
                      ((uint64_t)(GPIO_CRL_CNF0 | GPIO_CRL_MODE0) << 4*JOY_RIGHT) |
                      ((uint64_t)(GPIO_CRL_CNF0 | GPIO_CRL_MODE0) << 4*JOY_LEFT)) >> 0);
  // - set GPIO lines as floating inputs (i.e. conf 0x01 & mode 0x00)
  JOY_GPIO->CRH |= (((uint64_t)(GPIO_CRL_CNF0_0) << 4*JOY_OK) |
                    ((uint64_t)(GPIO_CRL_CNF0_0) << 4*JOY_UP) |
                    ((uint64_t)(GPIO_CRL_CNF0_0) << 4*JOY_DOWN) |
                    ((uint64_t)(GPIO_CRL_CNF0_0) << 4*JOY_RIGHT) |
                    ((uint64_t)(GPIO_CRL_CNF0_0) << 4*JOY_LEFT)) >> 32;
  JOY_GPIO->CRL |= (((uint64_t)(GPIO_CRL_CNF0_0) << 4*JOY_OK) |
                    ((uint64_t)(GPIO_CRL_CNF0_0) << 4*JOY_UP) |
                    ((uint64_t)(GPIO_CRL_CNF0_0) << 4*JOY_DOWN) |
                    ((uint64_t)(GPIO_CRL_CNF0_0) << 4*JOY_RIGHT) |
                    ((uint64_t)(GPIO_CRL_CNF0_0) << 4*JOY_LEFT)) >> 0;


  // DWT unit enable
  dwt_Init();


  // I2C interface configuration
  i2c_Init();


  // LCD HEM6432 configuration
  lcdHEM6432_Init();       // display hardware reset & configuration
  lcdHEM6432_Clean();      // clean display

  lcdHEM6432_GoToXY(22, 1);
  lcdHEM6432_PutText((uint8_t *)"LCD");
  lcdHEM6432_GoToXY(10, 2);
  lcdHEM6432_PutText((uint8_t *)"HEM6432");


  // SysTick configuration (systick period = 10ms)
  SysTick_Config(SYS_CLK/100);


  //
  // Program main loop
  //
  while (1){

    // Do every 0.5s
    if (F_500ms){
      F_500ms = 0;

      // LED1 blink
      if (LED_STATE(LED_1)) LED_ON(LED_1);
        else LED_OFF(LED_1);

      // HEM6432-03 lcd demo
      lcdDemo();
    }

    // Do every second
    if (F_1s){
      F_1s = 0;
    }
  }
}


/* LCD demo data -------------------------------------------------------------*/
// LCD demo basic step data
typedef struct{
  uint8_t   x_pos;           // frame begining column
  uint8_t   y_pos;           // frame beginning row
  uint8_t   demoChar;        // frame character
} lcdDemoStpData_TypeDef;

// LCD demo sequence
const lcdDemoStpData_TypeDef lcdDemoPattern[48] =
{
  {2,  0, '-'},
  {8,  0, '-'},
  {14, 0, '-'},
  {20, 0, '-'},
  {26, 0, '-'},
  {32, 0, '-'},
  {38, 0, '-'},
  {44, 0, '-'},
  {50, 0, '-'},
  {56, 0, '-'},
  {59, 1, '|'},
  {59, 2, '|'},
  {56, 3, '-'},
  {50, 3, '-'},
  {44, 3, '-'},
  {38, 3, '-'},
  {32, 3, '-'},
  {26, 3, '-'},
  {20, 3, '-'},
  {14, 3, '-'},
  {8,  3, '-'},
  {2,  3, '-'},
  {0,  2, '|'},
  {0,  1, '|'},

  {2,  0, ' '},
  {8,  0, ' '},
  {14, 0, ' '},
  {20, 0, ' '},
  {26, 0, ' '},
  {32, 0, ' '},
  {38, 0, ' '},
  {44, 0, ' '},
  {50, 0, ' '},
  {56, 0, ' '},
  {59, 1, ' '},
  {59, 2, ' '},
  {56, 3, ' '},
  {50, 3, ' '},
  {44, 3, ' '},
  {38, 3, ' '},
  {32, 3, ' '},
  {26, 3, ' '},
  {20, 3, ' '},
  {14, 3, ' '},
  {8,  3, ' '},
  {2,  3, ' '},
  {0,  2, ' '},
  {0,  1, ' '}
};
/*----------------------------------------------------------------------------*/

/**-----------------------------------------------------------------------------
  Name         :  lcdDemo
  Description  :  Simple lcd demo
                  Demo displays moving frame around the screen, which takes 24
                  steps. Then the frame is erased which takes the next 24 steps.
                  Every second screen of the demo is displayed in inverted mode.
  Argument(s)  :  none
  Return value :  none
------------------------------------------------------------------------------*/
void lcdDemo(void)
{
  static uint8_t frame_step = 0;       // frame step counter

  if (frame_step == 0){                // the first screen
    lcdHEM6432_SetNormalDisplay();
  }
  if (frame_step == 24){               // second demo screen
    lcdHEM6432_SetInvertedDisplay();
  }

  // Draw frame
  lcdHEM6432_GoToXY(lcdDemoPattern[frame_step].x_pos, lcdDemoPattern[frame_step].y_pos);
  lcdHEM6432_PutChar(lcdDemoPattern[frame_step].demoChar);

  frame_step++;                        // count demo steps
  if (frame_step >= 48){
    frame_step = 0;
  }
}


/**-----------------------------------------------------------------------------
  Name         :  Delay
  Description  :  Calibrated delay routine
                  Delay time is measured by the TimeDelay counter that is serviced
                  in SysTick handler routine
  Argument(s)  :  nTime: delay time length (in 10 ms)
  Return value :  none
------------------------------------------------------------------------------*/
void Delay(__IO uint32_t nTime)
{
  timingDelay = nTime;

  while(timingDelay != 0);
}


/**-----------------------------------------------------------------------------
  Name         :  SysTick_Handler
  Description  :  Function handles SysTick Handler
  Argument(s)  :  none
  Return value :  none
------------------------------------------------------------------------------*/
void SysTick_Handler(void)
{
  static uint8_t sys_timer = 0;

  // Service 0.5s and 1s flags
  sys_timer++;
  if (sys_timer == 50){
    F_500ms = 1;
  }
  if (sys_timer >= 100){
    sys_timer = 0;
    F_500ms = 1;
    F_1s = 1;
  }

  // Service calibrated delay function
  if (timingDelay != 0x00){
    timingDelay--;
  }
}


/**-----------------------------------------------------------------------------
  Name         :  SystemInit
  Description  :  System init for the STM32 Butterfly
                  Function configures different system clocks:
                      fREF = 10MHz (HSE)
                      fSYSCLK = 72MHz
                      fAPB1 = 36MHz
                      fAPB2 = 72MHz
  Argument(s)  :  none
  Return value :  none
------------------------------------------------------------------------------*/
void SystemInit(void)
{
  u16 StartUpCounter = 0;

  /* Setup the microcontroller system. Initialize the Embedded Flash Interface */

  /* Enable HSE=10MHz */
  RCC->CR |= RCC_CR_HSEON;

  /* Wait till HSE is ready and if Time out is reached exit */
  do {
    StartUpCounter++;
  } while(((RCC->CR & RCC_CR_HSERDY) == RESET) && (StartUpCounter != HSEStartUp_TimeOut));

  if (RCC->CR & RCC_CR_HSERDY){
    /* Enable Prefetch Buffer */
    FLASH->ACR |= FLASH_ACR_PRFTBE;
    /* Flash 2 wait state */
    FLASH->ACR = (FLASH->ACR & ~FLASH_ACR_LATENCY) | FLASH_ACR_LATENCY_2;

    /* HCLK = SYSCLK = 72MHz*/
    RCC->CFGR = (RCC->CFGR & ~RCC_CFGR_HPRE) | RCC_CFGR_HPRE_DIV1;
    /* PCLK2 = HCLK = 72MHz*/
    RCC->CFGR = (RCC->CFGR & ~RCC_CFGR_PPRE2) | RCC_CFGR_PPRE2_DIV1;
    /* PCLK1 = HCLK/2 = 36MHz*/
    RCC->CFGR = (RCC->CFGR & ~RCC_CFGR_PPRE1) | RCC_CFGR_PPRE1_DIV2;

    /* Configure PLLs ------------------------------------------------------*/
    /* PREDIV2 configuration: PREDIV2CLK = HSE / 2 = 5Hz */
    RCC->CFGR2 = (RCC->CFGR2 & ~RCC_CFGR2_PREDIV2) | RCC_CFGR2_PREDIV2_DIV2;

    /* PLL3 configuration: PLL3CLK = (HSE / 2) * 10 = 50 MHz */
    RCC->CFGR2 = (RCC->CFGR2 & ~RCC_CFGR2_PLL3MUL) | RCC_CFGR2_PLL3MUL10;
    /* Enable PLL3 */
    RCC->CR |= RCC_CR_PLL3ON;
    /* Wait till PLL3 is ready */
    while((RCC->CR & RCC_CR_PLL3RDY) == RESET);

    /* MCO configuration: MCO_OUT = PLL3 / 2 = 25MHz for PHY MII*/
    //RCC->CFGR = (RCC->CFGR ~(RCC_CFGR_MCO)) | RCC_CFGR_MCO_PLL3CLK_Div2;
    /* MCO disabled because PA8 is used as I/O pin*/
    RCC->CFGR = (RCC->CFGR & ~RCC_CFGR_MCO) | RCC_CFGR_MCO_NOCLOCK;

    /* PLL2 configuration: PLL2CLK = (HSE / 2) * 8 = 40 MHz */
    RCC->CFGR2 = (RCC->CFGR2 & ~RCC_CFGR2_PLL2MUL) | RCC_CFGR2_PLL2MUL8;
    /* Enable PLL2 */
    RCC->CR |= RCC_CR_PLL2ON;
    /* Wait till PLL2 is ready */
    while((RCC->CR & RCC_CR_PLL2RDY) == RESET);

    /* PREDIV1 configuration: PREDIV1CLK = PLL2 / 5 = 8 MHz */
   	RCC->CFGR2 = (RCC->CFGR2 & ~RCC_CFGR2_PREDIV1) | RCC_CFGR2_PREDIV1_DIV5 |
                  RCC_CFGR2_PREDIV1SRC_PLL2;
    /* PLL1 configuration: PLLCLK = PREDIV1 * 9 = 72 MHz */
    RCC->CFGR = (RCC->CFGR & ~RCC_CFGR_PLLMULL) | RCC_CFGR_PLLMULL9 |
                 RCC_CFGR_PLLSRC;
    /* Enable PLL1 */
    RCC->CR |= RCC_CR_PLLON;
    /* Wait till PLL1 is ready */
    while((RCC->CR & RCC_CR_PLLRDY) == RESET);

    /* Select PLL as system clock source */
    RCC->CFGR = (RCC->CFGR & ~RCC_CFGR_SW) | RCC_CFGR_SW_PLL;
    /* Wait till PLL is used as system clock source */
    while ((RCC->CFGR & RCC_CFGR_SWS) != RCC_CFGR_SWS_PLL);
  }
}

/*----------------------------- End of file ----------------------------------*/
